# { chr_name : [ [start, stop, depth], [start, stop, depth] ] }

import re, subprocess, os, tempfile, copy, pdb
from gr import Gr

CSTART = 0
CSTOP = 1
CDEPTH = 2
FORWARD = True
REVERSE = False
CYGWIN_PATH = "C:\\cygwin64\\bin\\"

NAME = 0
DEPTH = 1
BASES = 2
RANGE = 3
COMPONENTS = 4
DISEASES = 5
WEIGHTED_COMPONENTS = 6
COVERED = True
UNCOVERED = False
        
AM_NAME = 0
AM_FDEPTH = 1
AM_RDEPTH = 2
AM_GR = 3


class Cov(dict):


    @staticmethod
    def _amplicon_number(name):
        chrom, start = re.match("(chr[0-9XYM]+):([0-9]+)-", name[0]).groups()[0:2]
        return (Gr.KARYOTYPE[chrom]*Gr.MAX_CHR_LENGTH)+int(start)


    @classmethod
    def load_bam(coverage, bam, gr, amplicons=False):
        return coverage._load_bam_amplicons(bam, gr) if amplicons else  coverage._load_bam_exons(bam, gr)
        

    @classmethod
    def _load_bam_exons(coverage, bamfile_name, exons):
        cov = coverage()
        bedfile_temp = tempfile.TemporaryFile()
        exons.merged.save(bedfile_temp)
        bedfile_temp.seek(0, 0)

        bedtools = coverage._create_bedtools(bamfile_name, bedfile_temp, stranded=False)

        name = ""
        while True:
            splitline = bedtools.stdout.readline().rstrip("\n").split("\t")
            if splitline == [""] or splitline[3] != name:
                if name != "":
                    cov[chr_name].append([runstartpos, pos, depth])
                if splitline == [""]:
                    break
                chr_name = splitline[0]
                if chr_name not in cov:
                    cov[chr_name] = []
                runstartpos = int(splitline[1])+int(splitline[6])
                runstartdepth = int(splitline[7])
                cov[chr_name].append([runstartpos-1, runstartpos-1, 0])
        
            pos = int(splitline[1])+int(splitline[6])
            depth = int(splitline[7])
            if runstartdepth != depth:
                cov[chr_name].append([runstartpos, pos-1, runstartdepth])
                runstartpos = pos
                runstartdepth = depth
            name = splitline[3]
        bedfile_temp.close()
            
        for chr_name in cov:
            cov[chr_name].sort()
    
        for chr_name in cov:
            cov[chr_name][0][CSTART] = 0
            for index in range(1, len(cov[chr_name])):
                #if cov[chr_name][index][CSTART] != (cov[chr_name][index-1][CSTOP]+1) and (cov[chr_name][index][CSTART] != cov[chr_name][index][CSTOP] or cov[chr_name][index][CDEPTH] != 0):
                #    print cov[chr_name][index]###########################################????????????????????????????????????????????????????????????????????????????
                cov[chr_name][index][CSTART] = cov[chr_name][index-1][CSTOP]+1
            cov[chr_name].append([cov[chr_name][-1][CSTOP]+1, Gr.MAX_CHR_LENGTH, 0])
        return cov


    @classmethod
    def _load_bam_amplicons(coverage, bamfile_name, amplicons):
        cov = coverage()

        amplicon_metrics = {}
        for chr_name in amplicons:
            for entry in amplicons[chr_name]:
                amplicon_metrics[entry.name] = Amplicon_info([entry.name, "NA", "NA", Gr(entry)])

        bedfile_temp = tempfile.TemporaryFile()
        for chr_name in sorted(amplicons):
            if len(amplicons[chr_name]) == 1:
                bedfile_temp.write("{0}\t{2}\t{3}\t{1}\t.\t+\n{0}\t{2}\t{3}\t{1}\t.\t-\n".format(chr_name, amplicons[chr_name][0].name,
                    amplicons[chr_name][0].start-1, 
                    amplicons[chr_name][0].stop))

            else:
                bedfile_temp.write("{0}\t{2}\t{3}\t{1}\t.\t+\n{0}\t{4}\t{5}\t{1}\t.\t-\n".format(chr_name, amplicons[chr_name][0].name,
                    amplicons[chr_name][0].start-1, 
                    min(amplicons[chr_name][0].stop, amplicons[chr_name][1].start-1),
                    amplicons[chr_name][0].start-1, 
                    amplicons[chr_name][0].stop))

                for index in range(1, len(amplicons[chr_name])-1):
                    bedfile_temp.write("{0}\t{2}\t{3}\t{1}\t.\t+\n{0}\t{4}\t{5}\t{1}\t.\t-\n".format(chr_name, amplicons[chr_name][index].name,
                        amplicons[chr_name][index].start-1,
                        min(amplicons[chr_name][index].stop, amplicons[chr_name][index+1].start-1),
                        max(amplicons[chr_name][index].start, amplicons[chr_name][index-1].stop+1)-1, 
                        amplicons[chr_name][index].stop))

                bedfile_temp.write("{0}\t{2}\t{3}\t{1}\t.\t+\n{0}\t{4}\t{5}\t{1}\t.\t-\n".format(chr_name, amplicons[chr_name][-1].name,
                    amplicons[chr_name][-1].start-1, 
                    amplicons[chr_name][-1].stop,
                    max(amplicons[chr_name][-1].start, amplicons[chr_name][-2].stop+1)-1, 
                    amplicons[chr_name][-1].stop))
        bedfile_temp.seek(0, 0)

        bedtools = coverage._create_bedtools(bamfile_name, bedfile_temp, stranded=True)

        # loop one through bedtools output and transcribe positions and depths into "firstpass" 
        firstpass = {}# { "chr1" : [ [pos, depth], [pos, depth], [pos, depth], ...] }
        name = "" 
        while True:
            splitline = bedtools.stdout.readline().rstrip("\n").split("\t")
            if splitline == [""] or splitline[3] != name or splitline[5] != strand:
                if name != "":
                    if chr_name not in firstpass:
                        firstpass[chr_name] = []
                    firstpass[chr_name] += tempdata
                    if amplicon_metrics[name][(strand=="-")+1] != "NA":  
                        raise CoverMiError("Duplicate amplicon {0}".format(name))
                    amplicon_metrics[name][(strand=="-")+1] = tempdata[len(tempdata)/2][1] if (len(tempdata)>0) else 0
                        
                if splitline == [""]:
                    break
                tempdata = []
                chr_name = splitline[0]
        
            depth = int(splitline[7])
            strand = splitline[5]
            name = splitline[3]
            if depth > 0: 
                tempdata.append([int(splitline[1])+int(splitline[6]), depth])
        bedfile_temp.close()

        cov = coverage._firstpass_into_coverage(firstpass)
        cov.amplicon_info = amplicon_metrics.values()
        cov.amplicon_info.sort(key=coverage._amplicon_number)
        return cov


    @staticmethod
    def _create_bedtools(bamfile_name, bedfile_temp, stranded=False):
        if os.name == "nt":
            bamfile_name = os.path.abspath(bamfile_name).replace("C:", "/cygdrive/c").replace("\\", "/")
            def decorate(command): return "{0}bash.exe -c \"PATH=/usr/bin:/usr/local/bin:$PATH; {1}; exit\"".format(CYGWIN_PATH, command)
        else:
            def decorate(command): return command
        version = subprocess.Popen(decorate("bedtools --version"), stdout=subprocess.PIPE, universal_newlines=True, shell=True).stdout.readline().strip()
        if not version.startswith("bedtools v"):
            raise CoverMiError("Unable to start bedtools")
        major, minor, patch =  [int(num) for num in version[10:].split(".")]
        if major > 2 or (major == 2 and minor >=24):
            command = "bedtools coverage -d {0} -a '{2}' -b {1}"
        else:
            command = "bedtools coverage -d {0} -abam '{1}' -b {2}"
            
        command = decorate(command.format("-s" if stranded else "", bamfile_name, "stdin"))
        return subprocess.Popen(command, stdout=subprocess.PIPE, stdin=bedfile_temp, universal_newlines=True, shell=True)


    @classmethod
    def _firstpass_into_coverage(coverage, fp):
        cov = coverage()

        for chr_name in fp:
            fp[chr_name].sort()
            cov[chr_name] =[[0, 0, 0]]
            lastpos = 0

            for index in range(0,len(fp[chr_name])):
                if index < len(fp[chr_name])-2 and fp[chr_name][index][0] == fp[chr_name][index+1][0]:#if depth exists for adjacent + and - reads then combine
                    index += 1
                    fp[chr_name][index][1] += fp[chr_name][index-1][1]

                if fp[chr_name][index][0] > lastpos+1 and cov[chr_name][-1][CDEPTH] > 0:
                    cov[chr_name].append([lastpos+1, 0, 0])
                    
                if fp[chr_name][index][1] <> cov[chr_name][-1][CDEPTH]:
                    cov[chr_name].append([fp[chr_name][index][0], 0, fp[chr_name][index][1]])
                lastpos=fp[chr_name][index][0]

            if cov[chr_name][-1][CDEPTH] > 0:
                cov[chr_name].append([lastpos+1, 0, 0])
            cov[chr_name].append([Gr.MAX_CHR_LENGTH, Gr.MAX_CHR_LENGTH, 0])

        for chr_name in cov: #Loop over coverage object start positions and add in stop positions
            for index in range(0, len(cov[chr_name])-1):
                cov[chr_name][index][CSTOP] = cov[chr_name][index+1][CSTART]-1
        return cov


    @classmethod
    def perfect_coverage(coverage, gr1, perfect_depth=1000):
        firstpass = {}
        for chr_name in gr1:
            firstpass[chr_name] = []
            for entry in gr1[chr_name]:
                read_length = (entry.stop - entry.start + 1) * 2 / 3
                for pos in range(entry.start, entry.start+read_length+1):
                    firstpass[chr_name].append([pos, perfect_depth])
                for pos in range(entry.stop-read_length, entry.stop+1):
                    firstpass[chr_name].append([pos, perfect_depth])
        return coverage._firstpass_into_coverage(firstpass)


    @classmethod
    def load(coverage, path):
        cov = coverage()
        with file(path, "rU") as f:
            for row in f:
                chr_name, cstart, cstop, depth = row.rstrip("\n").split("\t")
                if chr_name not in cov:
                    cov[chr_name] = []
                cov[chr_name].append([int(cstart), int(cstop) ,int(depth)])
        return cov


    def __getitem__(self, chr_name):
        try:
            return super(type(self), self).__getitem__(chr_name)
        except KeyError:
            return [[0, Gr.MAX_CHR_LENGTH-1, 0], [Gr.MAX_CHR_LENGTH, Gr.MAX_CHR_LENGTH, 0]]


    def calculate(self, gr1, min_depth, exons=False, total=False):
        results = []
        resultkey = {}
        for entry in gr1.all_entries:
            name = entry.name if (not total) else ""
            name = name if (not exons) else "{0} e{1}".format(name, entry.exon)
            if name not in resultkey:
                resultkey[name] = len(results)
                results.append(Coverage_info([ name, [0, 0], [0, 0], [Gr(), Gr()], [0, 0], None, [0, 0] ]))
                try:
                    results[resultkey[name]][DISEASES] = entry.diseases
                except AttributeError:
                    pass
            line = results[resultkey[name]]

            allcovered = True
            for cstart, cstop, cdepth in self[entry.chrom]:
                if cstart > entry.stop:
                    break
                elif cstop >= entry.start:
                    bases = min(entry.stop, cstop) - max(entry.start, cstart) + 1
                    line[DEPTH][cdepth>=min_depth] += (bases*cdepth)
                    line[BASES][cdepth>=min_depth] += bases
                    allcovered = allcovered and (cdepth>=min_depth)
                    if entry.chrom in (line[RANGE][cdepth>=min_depth]) and cstart == line[RANGE][cdepth>=min_depth][entry.chrom][-1].stop+1:
                            line[RANGE][cdepth>=min_depth][entry.chrom][-1].stop = min(entry.stop, cstop)
                    else:
                        line[RANGE][cdepth>=min_depth].construct(copy.copy(entry)) 
                        line[RANGE][cdepth>=min_depth][entry.chrom][-1].start = max(entry.start, cstart) 
                        line[RANGE][cdepth>=min_depth][entry.chrom][-1].stop = min(entry.stop, cstop)
            line[COMPONENTS][COVERED] += int(allcovered)
            line[COMPONENTS][UNCOVERED] += int(not(allcovered))
            try:
                line[WEIGHTED_COMPONENTS][COVERED] += int(allcovered) * entry.weight
                line[WEIGHTED_COMPONENTS][UNCOVERED] += int(not(allcovered)) * entry.weight
            except AttributeError:
                pass

        for result in results:
            for index in [0,1]:
                result[DEPTH][index] /= max(result[BASES][index], 1)
        results.sort()
        return results if (not total) else results[0]


    def save_plot(self, gr1, filename): #Replaced by decorated plots but may be useful for debugging plotting
        with file(filename, "wt") as f:
            f.write("chrom\tpos\tdepth\tname\n")
            for chr_name in gr1:
                for entry in gr1[chr_name]:
                    f.write("{0}\t{1}\t{2}\t{3}\n".format(entry.chrom, entry.start-1, 0, entry.name))
                    for cstart, cstop, cdepth in self[chr_name]:
                        if cstop >= start:
                            f.write("{0}\t{1}\t{2}\t{3}\n".format(entry.chrom, max(entry.start, cstart), cdepth, entry.name))
                        if cstop >= stop:
                            if cstart < stop:
                                f.write("{0}\t{1}\t{2}\t{3}\n".format(entry.chrom, entry.stop, cdepth, entry.name))
                            break
                    f.write("{0}\t{1}\t{2}\t{3}\n".format(entry.chrom, entry.stop+1, 0, entry.name))


    def filter_by_germline(self, germ, germmin):
        newcov = type(self)()

        for chrom in self:

            newcov[chrom] = []
            selfindex = 0
            germindex = 0
            start = 0
            current = 0
            depth = 0

            selflist = self[chrom]
            germlist = germ[chrom]
            while True:
  
                newdepth = 0 if (germlist[germindex][CDEPTH] < germmin) else selflist[selfindex][CDEPTH]
                if depth != newdepth:
                    newcov[chrom].append([start, current-1, depth])
                    start = current
                    depth = newdepth

                if germlist[germindex][CSTOP] < selflist[selfindex][CSTOP]:
                    germindex += 1
                    current = germlist[germindex][CSTART]
                elif germlist[germindex][CSTOP] > selflist[selfindex][CSTOP]:
                    selfindex += 1
                    current = selflist[selfindex][CSTART]
                elif len(selflist) > selfindex+1:
                    selfindex += 1
                    germindex += 1
                    current = selflist[selfindex][CSTART]
                else:
                    newcov[chrom].append([start, selflist[selfindex][CSTOP], depth])
                    break
        return newcov


    def save(self, path):
        with file(path, "wt") as f:
            for chr_name in self:
                for entry in self[chr_name]:
                    f.write("{0}\t{1}\t{2}\t{3}\n".format(chr_name, entry[CSTART], entry[CSTOP], entry[CDEPTH]))


class Amplicon_info(list):
    @property
    def name(self):
        return self[AM_NAME]
    @property
    def chrom(self):
        return self[AM_NAME].split(":")[0]
    @property
    def f_depth(self):
        return self[AM_FDEPTH]
    @property
    def r_depth(self):
        return self[AM_RDEPTH]
    @property
    def minimum_depth(self):
        return self[AM_FDEPTH] if self[AM_FDEPTH]<self[AM_RDEPTH] else self[AM_RDEPTH]
    @property
    def maximum_depth(self):
        return self[AM_RDEPTH] if self[AM_FDEPTH]<self[AM_RDEPTH] else self[AM_FDEPTH]
    @property
    def mean_depth(self):
        return (self[AM_FDEPTH]+self[AM_RDEPTH])/2
    @property
    def ratio(self):
        return float(min(self[AM_FDEPTH], self[AM_RDEPTH]))/max(self[AM_FDEPTH], self[AM_RDEPTH], 1)
    @property
    def gr(self):
        return self[AM_GR]


class Coverage_info(list):
    @property
    def name(self):
        return self[NAME]
    @property
    def diseases(self):
        return self[DISEASES]
    @property
    def percent_covered(self):
        return float(self[BASES][COVERED]*100)/sum(self[BASES])
    @property
    def percent_uncovered(self):
        return float(self[BASES][UNCOVERED]*100)/sum(self[BASES])
    @property
    def depth_covered(self):
        return self[DEPTH][COVERED]
    @property
    def depth_uncovered(self):
        return self[DEPTH][UNCOVERED]
    @property
    def bases_covered(self):
        return self[BASES][COVERED]
    @property
    def bases_uncovered(self):
        return self[BASES][UNCOVERED]
    @property
    def range_covered(self):
        return self[RANGE][COVERED]
    @property
    def range_uncovered(self):
        return self[RANGE][UNCOVERED]
    @property
    def bases(self):
        return sum(self[BASES])
    @property
    def components_covered(self):
        return self[COMPONENTS][COVERED]
    @property
    def components_uncovered(self):
        return self[COMPONENTS][UNCOVERED]
    @property
    def components(self):
        return sum(self[COMPONENTS])
    @property
    def percent_components_covered(self):
        return float(self[COMPONENTS][COVERED]*100)/sum(self[COMPONENTS])
    @property
    def percent_components_uncovered(self):
        return float(self[COMPONENTS][UNCOVERED]*100)/sum(self[COMPONENTS])
    @property
    def weighted_components_covered(self):
        return self[WEIGHTED_COMPONENTS][COVERED]
    @property
    def weighted_components_uncovered(self):
        return self[WEIGHTED_COMPONENTS][UNCOVERED]
    @property
    def weighted_components(self):
        return sum(self[WEIGHTED_COMPONENTS])
    @property
    def percent_weighted_components_covered(self):
        return float(self[WEIGHTED_COMPONENTS][COVERED]*100)/sum(self[WEIGHTED_COMPONENTS])
    @property
    def percent_weighted_components_uncovered(self):
        return float(self[WEIGHTED_COMPONENTS][UNCOVERED]*100)/sum(self[WEIGHTED_COMPONENTS])
    @property
    def completely_covered(self):
        return self[BASES][UNCOVERED]==0
    @property
    def incompletely_covered(self):
        return self[BASES][UNCOVERED]>0





#!/usr/bin env python
from panel import Panel
import sys, os, tkFileDialog, Tkinter, re
import pdb




def main():
    chr_translate = { "23":"X", "24":"Y", "25":"M" }

    rootwindow = Tkinter.Tk()
    rootwindow.withdraw()

    print "Please select the COSMIC mutation tsv file"
    cosmic_path = tkFileDialog.askopenfilename(parent=rootwindow, title='Please select the COSMIC mutation file')
    if cosmic_path == "":
        sys.exit()
    if  len(file(cosmic_path, "rU").readline().split("\t")) != 29:
        print "File {0} is of incorrect format".format(os.path.basename(cosmic_path))
        sys.exit()

    print "Please select a panel"
    panel_path = tkFileDialog.askdirectory(parent=rootwindow, title='Please select a panel')
    if panel_path == "":
        sys.exit()

    panel = Panel(panel_path)
    if "Variants" in panel:
        print "Variants file {0} already exists within panel".format(os.path.basename(panel["Variants"]))
        sys.exit()
    if "Targets" not in panel:
        print "Unable to identify target gene file. Exiting"
        sys.exit()
    if "Disease_Names" in panel:
        print "Disease names file {0} will be deleted and updated".format(os.path.basename(panel["Disease_Names"]))
        if raw_input("Press y to continue, any other key to abort") != "y":
            sys.exit()

    genes = set([])
    with file(panel["Targets"] ,"rU") as f:
        for line in f:
            gene = line.strip().split()[0]
            if gene != "":
                genes.add(gene)

    outputfilepath = os.path.join(panel_path, 
                              "{0}_{1}.txt".format(os.path.basename(panel_path.rstrip(os.pathsep)), os.path.splitext(os.path.basename(cosmic_path.rstrip(os.pathsep)))[0]))
    if os.path.exists(outputfilepath):
        print "File {0} already exists".format(outputfilepath)
        sys.exit()

    print "Writing file {0}".format(os.path.basename(outputfilepath))
    with file(outputfilepath, "wt") as of:
        of.write("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}\n".format( 
                 "HGMD ID", "Disease", "Variant Class", "Gene Symbol", "chromosome", "coordinate start", "coordinate end", "strand", "hgvs"))

        diseases = {}
        with file(cosmic_path ,"rU") as f:
            for line in f:
                splitline = line.split("\t")
                cosmic_gene = splitline[0].split("_")[0]
                if cosmic_gene in genes and splitline[7] == "haematopoietic_and_lymphoid_tissue":
                    for disease in reversed(splitline[7:11]):
                        if disease != "NS":
                            break
                    diseases[disease] = ""
                    chrom, startstop = splitline[18].split(":")
                    if chrom in chr_translate:
                        chrom = chr_translate[chrom]
                    start, stop = startstop.split("-")
                    of.write("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}\n".format(
                             ".", disease, ".", cosmic_gene, "chr"+chrom, start, stop, splitline[19], splitline[14]))

        with file(cosmic_path ,"rU") as f:
            for line in f:
                splitline = line.split("\t")
                cosmic_gene = splitline[0].split("_")[0]
                if cosmic_gene not in genes and splitline[7] == "haematopoietic_and_lymphoid_tissue":
                    for disease in reversed(splitline[7:11]):
                        if disease != "NS":
                            break
                    if disease in diseases:
                        chrom, startstop = splitline[18].split(":")
                        if chrom in chr_translate:
                            chrom = chr_translate[chrom]
                        start, stop = startstop.split("-")
                        of.write("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}\n".format(
                                 ".", disease, ".", cosmic_gene, "chr"+chrom, start, stop, splitline[19], splitline[14]))

    if "Disease_Names" in panel:
        with file(panel["Disease_Names"]) as f:
            for l in f:
                if l[0] != "#":
                    old_name, new_name = l.split("=").strip()
                    if old_name in diseases:
                        diseases[old_name] = "="+new_name
        os.unlink(panel["Disease_Names"])

    diseasefilepath = "{0}_Disease_Names.txt".format(os.path.splitext(outputfilepath)[0])
    if os.path.exists(diseasefilepath):
        print "File {0} already exists".format(os.path.basename(diseasefilepath))
        sys.exit()

    print "Writing file {0}".format(os.path.basename(diseasefilepath))
    with file(diseasefilepath, "wt") as f:
        f.write("#Variants Disease Name Translation\n")
        for t in sorted(diseases.items()):
            f.write("".join(t)+"\n")    

    options = panel.read_options()
    options["VariantFrequency"] = "True"
    panel.write_options(options)

if __name__ == "__main__":
    main()





from pkg_resources import resource_string
import pdb
from gr import Gr
import subprocess, os, sys, re

r_path = "unknown"

class _Encode(object):
    def __init__(self, name, strand, adj):
        self.name = name
        self.strand = strand
        self.adj = adj

    def encode(self, x, y, component):
        for beginning, end, fixed, scaling in self.adj:
            if x >= beginning and x <= end:
                break
        rel_pos = x - beginning
        adjustment = fixed + (rel_pos - (rel_pos/scaling))
        return "{0}\t{1}\t{2}\t{3}\t{4}\t{5}\n".format(x, y, self.name, component, self.strand, adjustment)


def plot(coverage, panel, outputstem):
    
    global r_path
    if r_path == "unknown":
        if os.name ==  "nt":
            newest_time = 0
            r_path = "unavailable"
            R_STEM = "C:\\Program Files\\R"
            for root, dirnames, filenames in os.walk(R_STEM):
                if "Rscript.exe" in filenames:
                    exe_path = "{0}\\Rscript.exe".format(root)
                    mtime = os.path.getmtime(exe_path)
                    if mtime > newest_time:
                        newest_time = mtime
                        r_path = "\""+exe_path+"\" \"{0}\""
        else:
            try:
                with file(os.devnull, "wt") as DEVNULL:
                    subprocess.check_call(["which", "R"], stdout=DEVNULL, stderr=DEVNULL)
                r_path = "R -f '{0}'"
            except CalledProcessError:
                r_path = "unavailable"
        if r_path == "unavailable":
            print "WARNING R binary not found. no graphs will be plotted"

    if "Transcripts" not in panel or r_path == "unavailable":
        return

    output_file = outputstem+"_plot.pdf"
    plotname = panel["Filenames"]["Sample"] if ("Sample" in panel["Filenames"]) else panel["Filenames"]["Panel"]
    rdataframe = outputstem+"_Rdata.tsv"
    with file(rdataframe, "wt") as f:
        s = "{0}\t{1}\t{2}\t{3}\t{4}\t{5}\n"
        f.write(s.format("x", "y", "name", "component", "strand", "adjustment"))

        for entry in panel["Transcripts"].all_entries:
                chrom = entry.chrom
                name = entry.name
                strand = entry.strand
                gr1 = Gr(entry)

                if "Amplicons" in panel:
                    gr1, amplicons = gr1.extended_to_include_touching_amplicons(panel["Amplicons"])
                else:
                    amplicons = Gr()
                auc = 0
                for amplicon in amplicons.all_entries:
                    auc += amplicon.stop - amplicon.start + 1                
                INTRON_SIZE = auc / amplicons.number_of_components if (amplicons.number_of_components>0) else 200

                start = gr1[chrom][0].start
                stop = gr1[chrom][0].stop
                
                exons = panel["Exons"].touched_by(gr1).subset2(name) if ("Exons" in panel) else Gr()

                blocks = amplicons.combined_with(exons).merged
                # adj = [start, stop, fixed subtraction, scaling factor]
                adj = []
                end_of_prev_block = start-2
                fixed_total = 0
                for bentry in blocks.all_entries:
                    bstart = bentry.start
                    bstop = bentry.stop
                    intron_size = bstart - end_of_prev_block - 1
                    adj.append((end_of_prev_block+1, bstart-1, fixed_total, max(float(intron_size)/INTRON_SIZE, 1)))
                    fixed_total += max(intron_size - INTRON_SIZE, 0)
                    adj.append((bstart, bstop, fixed_total, 1))
                    end_of_prev_block = bstop
                adj.append((end_of_prev_block+1, stop+1, fixed_total, max(float(stop - end_of_prev_block)/INTRON_SIZE, 1) if (not blocks.is_empty) else 1))

                line = _Encode(name, strand, adj)

                if "Amplicons" in panel:
                    for aentry in amplicons.all_entries:
                        f.write(line.encode(aentry.start, aentry.stop, "amplicon"))
                   
                f.write(line.encode(start-1, 0, "coverage"))
                for cstart, cstop, cdepth in coverage[chrom]:
                    if cstop >= start:
                        f.write(line.encode(max(start, cstart), cdepth, "coverage"))
                        if cstop >= stop:

                            f.write(line.encode(min(stop, cstop), cdepth, "coverage"))
                            break
                f.write(line.encode(stop+1, 0, "coverage"))

                if "Exons" in panel:
                    for eentry in exons.all_entries:
                        f.write(line.encode(eentry.start, eentry.stop, "exon"))
                        f.write(line.encode((eentry.start+eentry.stop)/2, str(eentry.exon), "exon_number"))
        
                if "Variants_Mutation" in panel:
                    variants = gr1.overlapped_by(panel["Variants_Mutation"])
                    vycoord = panel["Options"]["Depth"] if ("Depth" in panel["Options"]) else 0
                    for ventry in variants.all_entries:
                            f.write(line.encode((ventry.start+ventry.stop)/2, vycoord, "variants"))
        
                if "Depth" in panel["Options"]:
                    f.write(line.encode(start, panel["Options"]["Depth"], "minimum"))
    
    genericrcode = resource_string(__name__, "covermiplot.R")
    rscript = outputstem+"R"
    with file(rscript, "wt") as f:
        f.write(genericrcode.replace("INPUT", os.path.abspath(rdataframe)).replace("OUTPUT", os.path.abspath(output_file)).replace("NAME", plotname).replace("\\", "/"))
    
    with file(os.devnull, "wt") as DEVNULL:
        subprocess.call(r_path.format(rscript), shell=True, stdout=DEVNULL)
    
    os.unlink(rdataframe)
    os.unlink(rscript)





import os, pdb


class CoverMiException(Exception):
    pass


class Files(list):

    def __init__(self, path, ext, strip_trailing_underscore=True):

        self._strip_trailing_underscore = strip_trailing_underscore

        if os.path.isfile(path):
            run, sample = os.path.split(path)
            run = os.path.basename(run)
            sample, ex = os.path.splitext(sample)
            if ex != ext:
                raise CoverMiException("file "+path+" is not of type "+ext)
            if self._strip_trailing_underscore:
                sample = sample.split("_")[0]
            self.append((run, sample, path))

        elif os.path.isdir(path): 
            if not hasattr(ext, "__iter__"):
                ext = (ext,)

            for root, dirnames, filenames in os.walk(path):
                break
            layout = "multipleruns"
            for filename in filenames:
                if os.path.splitext(filename)[1] in ext:
                    layout = "singlerun"
                    break
        
            if layout == "singlerun":
                self._walk_samples(path, ext[0])
            elif layout == "multipleruns":
                self._walk_runs(path, ext[0])

            for ex in ext[1:len(ext)]:
                if self != type(self)(path, ex, self._strip_trailing_underscore):
                    raise CoverMiException(ext[0]+" and "+ex+" files do not match")#?Too strict

        else:
            raise CoverMiException(path+" is not a file or directory")


    def _walk_runs(self, path, ext):
        for root, dirnames, filenames in os.walk(path):
            break
        for run in dirnames:
            self._walk_samples(os.path.join(root, run), ext)


    def _walk_samples(self, path, ext):
        found = set([])
        run = os.path.basename(path)
        for root, dirnames, filenames in os.walk(path):
            for filename in filenames:
                filename, extension = os.path.splitext(filename)
                if extension == ext:
                    sample = filename.split("_")[0] if self._strip_trailing_underscore else filename
                    if (run, sample) in found:
                        raise CoverMiException("Sample "+sample+" duplicated in run "+run)
                    found.add((run, sample))
                    self.append((run, sample, os.path.join(root, filename)))









